/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is SpellBound code.
 *
 * The Initial Developer of the Original Code is
 * Robert Strong <moz@exchangecode.com>.
 * Portions created by the Initial Developer are Copyright (C) 2004-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function sbGetNodeType(node) {
  if (!node || !node.nodeName)
    return 0;

  if (node.nodeName == "editor") // hack to allow passed in editor elements
    return 202;

  // TODO: find a way to clean up the following try statements
  try {
    if (node.hasAttribute("id"))
      if (node.getAttribute("id") == "urlbar"
      || node.getAttribute("id") == "search-box"
      || node.getAttribute("id") == "sidebar-search-text")
        return 0;
  } catch (e) { }

  try {
    if (node.parentNode && node.parentNode.parentNode
    && node.parentNode.parentNode.parentNode
    && node.parentNode.parentNode.parentNode.hasAttribute("id"))
      if (node.parentNode.parentNode.parentNode.getAttribute("id") == "urlbar")
        return 0;
  } catch (e) { }

  try {
    if (node.parentNode
    && node.parentNode.parentNode
    && node.parentNode.parentNode.hasAttribute("id"))
      if (node.parentNode.parentNode.getAttribute("id") == "sidebar-search-text")
        return 0;
  } catch (e) { }

  try {
    if (node.hasAttribute("type"))
      if (node.getAttribute("type").toLowerCase() != "text")
        return 0;
  } catch (e) { }

  switch(node.nodeName.toLowerCase()) {
    case "textarea":      // textarea - duh
      return 100;
    case "input":         // input - duh
      return 101;
    case "textbox":       // textbox for extensions and chrome
      return 102;
    case "html:textarea": // html:textarea for extensions and chrome
      return 103;
    case "html:input":    // html:input for extensions and chrome
      return 104;
    default:              // editor or midas
      if (typeof node.ownerDocument.designMode != "undefined"
      && node.ownerDocument.designMode.toLowerCase() == "on") {
        if (node.ownerDocument.defaultView.frameElement) {
          if (node.ownerDocument.defaultView.frameElement.nodeName.toLowerCase() == "editor")
            return 200;   // editor
        }
        return 201;       // midas
      }

//XXX the following is a hack to detect editors when node.ownerDocument.defaultView.frameElement
// is not available and will be fixed in the next release
      if (document.getElementById("sidebar-box").collapsed ||
      document.getElementById("sidebar-box").hidden)
        break;

      if (node.ownerDocument.defaultView && !node.ownerDocument.defaultView.frameElement) {
        var sbSideBar = document.getElementById("sidebar");
        if (!sbSideBar)
        {
          sbSideBar = window.sidebarObj.panels
             .get_panel_from_id(window.SidebarGetLastSelectedPanel())
             .get_iframe();
        }
        if (sbSideBar)
        {
          var editors = sbSideBar.contentDocument.getElementsByTagName("editor");
          if (editors[0]) {
            var elems = editors[0].contentDocument.getElementsByTagName(node.tagName);
            for(var i = 0; i < elems.length; i++)
            {
              if (elems[i] == node)
              {
                if (sbOverlay && sbOverlay.lastFocusedNode)
                  sbOverlay.lastFocusedNode = editors[0]; // force lastFocusedNode to be the editor
                return 200;
              }
            }
          }
        }
      }
  }

  return 0;               // unsupported
}

function getSBPrefs() {
  return SpellBoundPrefs;
}

var SpellBoundPrefs = {
  preferences:  null,
  branch:      "spellbound.",

  getPreferences: function() {
    if (this.preferences)
      return this.preferences;

    try {
      this.preferences = Components.classes["@mozilla.org/preferences;1"].
                         getService(Components.interfaces.nsIPref);
    }
    catch (e) { dump("failed to get nsIPref!\n"); }

    return this.preferences;
  },

  setBoolPref: function(name, value) {
    if (this.getPreferences().GetBoolPref(this.branch + name) != value)
      this.getPreferences().SetBoolPref(this.branch + name, value);
  },

  setCharPref: function(name, value) {
    if (this.getPreferences().GetCharPref(this.branch + name) != value)
      this.getPreferences().SetCharPref(this.branch + name, value);
  },

  setIntPref: function(name, value) {
    if (this.getPreferences().GetIntPref(this.branch + name) != value)
      this.getPreferences().SetIntPref(this.branch + name, value);
  },

  getBoolPref: function(name, defaultValue) {
    try {
      return this.getPreferences().GetBoolPref(this.branch + name);
    } catch (e) { }
    if(typeof defaultValue != "undefined") {
      try {
        this.getPreferences().SetDefaultBoolPref(this.branch + name, defaultValue)
        return defaultValue;
      } catch (e) { return defaultValue; }
    }
    return null;
  },

  getCharPref: function(name, defaultValue) {
    try {
      return this.getPreferences().GetCharPref(this.branch + name);
    } catch (e) { }
    if(typeof defaultValue != "undefined") {
      try {
        this.getPreferences().SetDefaultCharPref(this.branch + name, defaultValue)
        return defaultValue;
      } catch (e) { return defaultValue; }
    }
    return null;
  },

  getIntPref: function(name, defaultValue) {
    try {
      return this.getPreferences().GetIntPref(this.branch + name);
    } catch (e) { }
    if(typeof defaultValue != "undefined") {
      try {
        this.getPreferences().SetDefaultIntPref(this.branch + name, defaultValue)
        return defaultValue;
      } catch (e) { return defaultValue; }
    }
    return null;
  }
}
